unit CreateDatabase;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, UITypes,
  Dialogs, StdCtrls, Spin, DB, ADODB;

type
  TFormCreateDatabase = class(TForm)
    GroupBoxContainer: TGroupBox;
    ButtonClose: TButton;
    ButtonCreateDatabase: TButton;
    ButtonSelectDatabase: TButton;
    EditDatabaseName: TEdit;
    EditDatabaseFilename: TEdit;
    SpinEditDatabaseSize: TSpinEdit;
    LabelDataBaseSize: TLabel;
    LabelDataBaseFileName: TLabel;
    LabelDatabaseName: TLabel;
    ADOQuery: TADOQuery;
    ADOConnection: TADOConnection;
    SaveDialog: TSaveDialog;
    procedure ButtonSelectDatabaseClick(Sender: TObject);
    procedure ButtonCreateDatabaseClick(Sender: TObject);
    procedure ButtonCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    CreatedNewDataBase: Boolean;
  end;

var
  FormCreateDatabase: TFormCreateDatabase;

implementation

uses
    Main,
    Settings,
    SpecialFolders;

{$R *.dfm}

function GetDocumetsFolder: String;
var
    SettingsFolder: String;
begin
    try
        SettingsFolder := GetSpecialFolder(CSIDL_PERSONAL);
        SettingsFolder := SettingsFolder + '\3delite\Registration Tool\';
        ForceDirectories(SettingsFolder);
        Result := SettingsFolder;
    except
        //*
    end;
end;

procedure TFormCreateDatabase.ButtonCloseClick(Sender: TObject);
begin
    Close;
end;

procedure TFormCreateDatabase.ButtonCreateDatabaseClick(Sender: TObject);
var
    DialogResult: Integer;
begin

    Screen.Cursor := crAppStart;

    try
        ADOConnection.Close;

        ADOConnection.ConnectionString := FormSettings.EditMySQLConnectionString.Text;

        ADOQuery.SQL.Clear;

        ADOQuery.SQL.Add('CREATE DATABASE "' + EditDatabaseName.Text + '"');
        ADOQuery.SQL.Add('ON');
        ADOQuery.SQL.Add('(');
        ADOQuery.SQL.Add('NAME = "' + EditDatabaseName.Text + '_dat",');
        ADOQuery.SQL.Add('FILENAME = "' + EditDatabaseFilename.Text + '",');
        ADOQuery.SQL.Add('SIZE = ' + IntToStr(SpinEditDatabaseSize.Value));
        ADOQuery.SQL.Add(')');

        ADOQuery.ExecSQL;

        ADOQuery.SQL.Clear;

        //* Set to use our new database (specified by name)
        ADOQuery.SQL.Add('USE ' + EditDatabaseName.Text + ';');
        //* Create a new table
        ADOQuery.SQL.Add('CREATE TABLE "' + TN_SETTINGS + '"');
        ADOQuery.SQL.Add('(');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_ID + '" INT IDENTITY(1,1) NOT NULL,');
        //* Add the 2 specified columns as unicode type, max. 50 characters
        ADOQuery.SQL.Add('"' + CN_SETTINGS_VERSION + '" INT NOT NULL,');
        //ADOQuery.SQL.Add(CN_PROJECT_TABLENAME + ' VARCHAR(255) NOT NULL,');
        //ADOQuery.SQL.Add(CN_PROJECT_REG_SEED + ' VARCHAR(255) NOT NULL,');
        //ADOQuery.SQL.Add(CN_PROJECT_ICON + ' VARBINARY(MAX),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_DEFAULT_EMAIL_TEMPLATE_REGISTRATION + '" NTEXT,');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_DEFAULT_EMAIL_TEMPLATE_VERSIONCODE + '" NTEXT,');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_DEFAULT_EMAIL_TEMPLATE_VERSIONCODE_ERROR + '" NTEXT,');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_CHECK_INTERVAL + '" INT,');
        //* Incoming
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_GUID + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_ItemName + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_Active + '" BIT,');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_HostName + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_Port + '" INT,');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_LoginName + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_Password + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_Security + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_SSLMethod + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_ServerType + '" NVARCHAR(255),');

        //* Incoming version code requests
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_VERCODE_GUID + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_VERCODE_ItemName + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_VERCODE_Active + '" BIT,');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_VERCODE_HostName + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_VERCODE_Port + '" INT,');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_VERCODE_LoginName + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_VERCODE_Password + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_VERCODE_Security + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_VERCODE_SSLMethod + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_INCOMING_VERCODE_ServerType + '" NVARCHAR(255),');

        //* Outgoing
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_OUTGOING_GUID + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_OUTGOING_ItemName + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_OUTGOING_Active + '" BIT,');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_OUTGOING_HostName + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_OUTGOING_Port + '" INT,');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_OUTGOING_LoginName + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_OUTGOING_Password + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_OUTGOING_Security + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_OUTGOING_SSLMethod + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_SETTINGS_EMAIL_OUTGOING_ServerType + '" NVARCHAR(255),');
        //* Emails
        ADOQuery.SQL.Add('"' + CN_EMAIL_SENDER_ADDRESS + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_EMAIL_FORWARD_ADDRESS + '" NVARCHAR(255),');
        ADOQuery.SQL.Add('"' + CN_EMAIL_SUPPORT_ADDRESS + '" NVARCHAR(255)');
        //* PK
        ADOQuery.SQL.Add('PRIMARY KEY (' + CN_SETTINGS_ID + ')');
        ADOQuery.SQL.Add(')');

        ADOQuery.ExecSql;

        ADOQuery.SQL.Clear;

        //* Set to use our new database (specified by name)
        ADOQuery.SQL.Add('USE ' + EditDatabaseName.Text + ';');
        //* Create a new table
        ADOQuery.SQL.Add('CREATE TABLE "' + TN_PROJECTS + '"');
        ADOQuery.SQL.Add('(');
        ADOQuery.SQL.Add(CN_PROJECT_ID + ' INT IDENTITY(1,1) NOT NULL,');
        //* Add the 2 specified columns as unicode type, max. 50 characters
        ADOQuery.SQL.Add(CN_PROJECTNAME + ' NVARCHAR(255) NOT NULL,');
        ADOQuery.SQL.Add(CN_PROJECT_TABLENAME + ' VARCHAR(255) NOT NULL,');
        ADOQuery.SQL.Add(CN_PROJECT_REG_SEED + ' VARCHAR(255) NOT NULL,');
        ADOQuery.SQL.Add(CN_PROJECT_ICON + ' VARBINARY(MAX),');
        ADOQuery.SQL.Add(CN_PROJECT_EMAIL_TEMPLATE_REGISTRATION + ' NTEXT,');
        ADOQuery.SQL.Add(CN_PROJECT_EMAIL_TEMPLATE_VERSIONCODE + ' NTEXT,');
        ADOQuery.SQL.Add(CN_PROJECT_EMAIL_TEMPLATE_VERSIONCODE_ERROR + ' NTEXT,');
        ADOQuery.SQL.Add(CN_PROJECT_DOWNLOAD_URL + ' NTEXT');

        {
                    ADOQuery.SQL.Add('"' + CN_USER_SALUTATION + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_TITLE + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_LASTNAME + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_FIRSTNAME + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_COMPANY + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_CONTACTNAME + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_ADDRESS + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_CITY + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_FULLCITY + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_COUNTRY + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_POSTALCODE + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_STATEPROVINCE + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_TELEPHONE + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_MOBILEPHONE + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_FAX + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_HOMEPAGE + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_NEWSLETTER + '" BIT,');
                    ADOQuery.SQL.Add('"' + CN_USER_VATID + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_NOOFLICENSES + '" INT,');
                    ADOQuery.SQL.Add('"' + CN_USER_LANGUAGE + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_SHAREITREFNO + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_RESELLER + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_PROMOTIONNAME + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_NETSALESCURRENCY + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_NETSALES + '" MONEY,');
                    ADOQuery.SQL.Add('"' + CN_USER_NETDISCOUNTCURRENCY + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_NETDISCOUNT + '" MONEY,');
                    ADOQuery.SQL.Add('"' + CN_USER_COLLECTEDVATCURRENCY + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_COLLECTEDVAT + '" MONEY,');
                    ADOQuery.SQL.Add('"' + CN_USER_SHIPPINGCURRENCY + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_SHIPPING + '" MONEY,');
                    ADOQuery.SQL.Add('"' + CN_USER_VATSHIPPINGCURRENCY + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_VATSHIPPING + '" MONEY,');
                    ADOQuery.SQL.Add('"' + CN_USER_TOTALCURRENCY + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_TOTAL + '" MONEY,');
                    ADOQuery.SQL.Add('"' + CN_USER_PAYMENT + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_DELIVERYMODE + '" NVARCHAR(255),');
                    ADOQuery.SQL.Add('"' + CN_USER_COMMENTS + '" NTEXT');
        }
        ADOQuery.SQL.Add('PRIMARY KEY (' + CN_PROJECT_ID + ')');
        ADOQuery.SQL.Add(')');

        ADOQuery.ExecSql;

        ADOQuery.SQL.Clear;

        //* Set to use our new database (specified by name)
        ADOQuery.SQL.Add('USE ' + EditDatabaseName.Text + ';');
        //* Create a new table
        ADOQuery.SQL.Add('CREATE TABLE "' + TN_PROJECTS_SUBVERSION + '"');
        ADOQuery.SQL.Add('(');
        ADOQuery.SQL.Add(CN_PROJECT_ID + ' INT NOT NULL,');
        //* Add the 2 specified columns as unicode type, max. 50 characters
        ADOQuery.SQL.Add(CN_SUBVERSION_NAME + ' NVARCHAR(255) NOT NULL,');
        ADOQuery.SQL.Add(CN_SUBVERSION_VERCODE_SEED + ' VARCHAR(255) NOT NULL,');
        //ADOQuery.SQL.Add(RN_PROJECT_REG_SEED + ' VARCHAR(255) NOT NULL,');
        //ADOQuery.SQL.Add('PRIMARY KEY (Id)');
        ADOQuery.SQL.Add(')');

        ADOQuery.ExecSql;


        FormTool.CheckToUpgradeDataBaseToVersion3(EditDatabaseName.Text);
        FormTool.CheckToUpgradeDataBaseToVersion4(EditDatabaseName.Text);
        FormTool.CheckToUpgradeDataBaseToVersion5(EditDatabaseName.Text);
        FormTool.CheckToUpgradeDataBaseToVersion6(EditDatabaseName.Text);
        FormTool.CheckToUpgradeDataBaseToVersion7(EditDatabaseName.Text);

        //* Set default registration e-mail templates
        FormTool.SQLSetDefaultEMailTemplates(EditDatabaseName.Text, True, FormTool.MemoDefaultEMailTemplateRegistration.Text, FormTool.MemoDefaultEMailTemplateVersionCode.Text, FormTool.MemoDefaultEMailTemplateVersionCodeError.Text);

        if FileExists(EditDatabaseFilename.Text) then begin
            CreatedNewDataBase := True;
            DialogResult := MessageDlg('Successfully created new database "' + EditDatabaseName.Text + '".' + #13#10
                + 'Set to use this database?'
                , mtConfirmation, [mbYes, mbNo], 0);
            if DialogResult = mrYes then begin
                FormSettings.EditDatabaseNameToUse.Text := EditDatabaseName.Text;
                FormSettings.SaveSettings;
            end;
        end else begin
            MessageDlg('Error while creating new database ' + EditDatabaseName.Text + '.', mtError, [mbOK], 0);
        end;
    except
        on E: Exception do begin
            Screen.Cursor := crDefault;
            MessageDlg('Error: ' + E.Message, mtError, [mbOk], 0);
        end;
    end;
    Screen.Cursor := crDefault;
end;

procedure TFormCreateDatabase.ButtonSelectDatabaseClick(Sender: TObject);
begin
    if SaveDialog.Execute then begin
        EditDatabaseFilename.Text := SaveDialog.FileName;
    end;
end;

procedure TFormCreateDatabase.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    if CreatedNewDataBase then begin
        FormTool.ReloadProjectsWithExceptionHandling;
    end;
    CreatedNewDataBase := False;
end;

procedure TFormCreateDatabase.FormCreate(Sender: TObject);
begin
    EditDatabaseFilename.Text := GetDocumetsFolder +  'RegistrationDatabase.mdf';
end;

end.
